<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 09/04/20
 * Time: 11.02
 */

namespace FiloBlu\Rma\Helper;
use Magento\Framework\App\Helper\Context;
use Magento\Quote\Model\Quote\Address\RateRequestFactory;
use Magento\Sales\Api\Data\OrderAddressInterface;
use Magento\Store\Api\Data\StoreInterface;
use FiloBlu\Rma\Model\Carrier\MatrixrateFactory;
use Magento\Framework\DataObjectFactory;

class EasyReturnHelper extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var RateRequestFactory
     */
    private $rateRequestFactory;
    /**
     * @var MatrixrateFactory
     */
    private $matrixrateFactory;
    /**
     * @var DataObjectFactory
     */
    private $objectFactory;


    const EASY_RETURN_ENABEl_XML_PATH = 'filoblu_disablerma_section/easy_return_matrixrate_group/easy_return_enable';

    /**
     * EasyReturnHelper constructor.
     * @param RateRequestFactory $rateRequestFactory
     * @param MatrixrateFactory $matrixrateFactory
     * @param DataObjectFactory $objectFactory
     * @param Context $context
     */
    public function __construct(
        RateRequestFactory $rateRequestFactory,
        MatrixrateFactory $matrixrateFactory,
        DataObjectFactory $objectFactory,
        Context $context){
        $this->rateRequestFactory = $rateRequestFactory;
        $this->matrixrateFactory = $matrixrateFactory;
        $this->objectFactory = $objectFactory;
        parent::__construct($context);
    }


    /**
     * @param OrderAddressInterface $address
     * @param StoreInterface $store
     * @param float $weight
     * @return mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function simulateEasyReturnShippingCost(OrderAddressInterface $address, StoreInterface $store, float $weight) {

        $shipping_amounts [] = 0;
        $request = $this->rateRequestFactory->create();
        $request->setDestCountryId($address->getCountryId());
        $request->setDestRegionId($address->getRegionId());
        $request->setDestPostcode($address->getPostcode());
        $request->setDestCity($address->getCity());
        $request->setPackageWeight($weight);
        $request->setFreeMethodWeight($weight);
        $request->setPackageQty(1);
        $request->setStoreId($store->getId());
        $request->setWebsiteId($store->getWebsiteId());
        $request->setLimitCarrier(null);
        $request->setFreeShipping(null);
        //get rates for the matrixrate request created
        $matrixrate = $this->matrixrateFactory->create();
        $result = $matrixrate->collectRates($request);
        if ($result) {
            $rates = $result->getAllRates();
            if (!empty($rates)) {
                foreach ($rates as $rate) {
                    $price=$rate->getPrice();
                    if (isset($price) && $price > 0) {
                        $shipping_amounts[] = $price;
                    }
                }
            }
        }
        return max($shipping_amounts);
    }

    /**
     * @param null $storeId
     * @return bool
     */
    public function isEasyReturnEnabled($storeId = null){
        return $this->scopeConfig->getValue(self::EASY_RETURN_ENABEl_XML_PATH,\Magento\Store\Model\ScopeInterface::SCOPE_STORE,$storeId) ?? false;
    }

}