<?php
namespace FiloBlu\Rma\Observer;
use FiloBlu\Rma\Helper\ChangeSizeAttributeHelper;
use FiloBlu\Rma\Helper\ChangeSizeOrder;
use FiloBlu\Rma\Logger\Logger;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Sales\Api\OrderRepositoryInterface;
use FiloBlu\Rma\Helper\SerializerHelper;

class UpdateStatusVerify implements ObserverInterface
{
    /**
     * @var Logger
     */
    private $logger;
    /**
     * @var ChangeSizeOrder
     */
    private $changeSizeOrder;
    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;

    const ALLOWED_ORDER_STATUS_PATH='filoblu_disablerma_section/rma_group/allowed_change_size_order_status';
    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;
    /**
     * @var SerializerHelper
     */
    private $serializerHelper;

    /**
     * UpdateStatusVerify constructor.
     * @param Logger $logger
     * @param ChangeSizeOrder $changeSizeOrder
     * @param OrderRepositoryInterface $orderRepository
     * @param ScopeConfigInterface $scopeConfig
     * @param SerializerHelper $serializerHelper
     */
    public function __construct(
        Logger $logger,
        ChangeSizeOrder $changeSizeOrder,
        OrderRepositoryInterface $orderRepository,
        ScopeConfigInterface $scopeConfig,
        SerializerHelper $serializerHelper){
        $this->logger = $logger;
        $this->changeSizeOrder = $changeSizeOrder;
        $this->orderRepository = $orderRepository;
        $this->scopeConfig = $scopeConfig;
        $this->serializerHelper = $serializerHelper;
    }


    /**
     * @param Observer $observer
     * @throws LocalizedException
     */
    public function execute(Observer $observer)
    {
        $order = $observer->getEvent()->getOrder();
        if ($order && $order->getId() && $this->changeSizeOrder->isChangeSizeOrder($order->getId())) {
            $toStatus = $order->getData("status");
            $fromStatus = $order->getOrigData("status");
            $fromState = $order->getOrigData("state");
            $allowedStatus = $this->scopeConfig->getValue(self::ALLOWED_ORDER_STATUS_PATH);
            // Added compatibility
            if (!is_array($allowedStatus)){
                // Then we need to unserialize the value
                $allowedStatus = $this->serializerHelper->unserialize($allowedStatus);
            }
            //$allowedStatus = ["order_exported", ChangeSizeAttributeHelper::ORDER_STATUS_CHANGESIZE_CODE,"pending","canceled","confirmed","confirmed_by_ext","payment_confirmed","payment_confirmed_on_ext"];
            $allowed=false;
            foreach ($allowedStatus as $item){
                if($item['allowed_status'] == $toStatus){
                    $allowed=true;
                    break;
                }
            }

            if (!$allowed && $toStatus != "order_exported") {
                if (!$this->changeSizeOrder->isValidForUpdateStatus($order->getId())) {
                    $this->logger->debug("FiloBlu\Rma\Observer: Order Status cannot be updated before linked RMA is processed .Order: " . $order->getIncrementId() . " is restored in Status Order Exported");

                        $this->logger->debug("FiloBlu\Rma\Observer: Order " . $order->getIncrementId() . " is restored to Order Exported Status.");
                        $order->setStatus("order_exported");
                        $order->setState("processing");
                        $order->setChangeSizeRestored(true);
                        $order->addStatusHistoryComment("Order is restored to Order Exported programmatically because linked RMA is still processing.");
                        $this->orderRepository->save($order);

                    if(!$order->getChangeSizeRestored()){
                        throw new LocalizedException(__("Status update not allowed!"));
                    }
                }
            }
        }
    }

}